#!/usr/bin/php4 -q
<?php
/*
GOpdc: populate.php4 - Create initial LDAP-tree for later use with GOpdc 

Authors: Jan Wenzel    <jan.wenzel@GONICUS.de>

Copyright (C) 2005 GONICUS GmbH

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Contact information: GONICUS GmbH
Moehnestrasse 11-17
D-59755 Arnsberg
Germany
tel: ++49 2932 916 0
fax: ++49 2932 916 230
email: info@GONICUS.de
http://www.GONICUS.de
*/

error_reporting(E_ALL);
//error_reporting(0);

// Read configuration
require_once("/usr/share/gopdc/config.inc");

// Map Options from XML-File to local var
/** The primary domain for which GOpdc should alter entries */
$smb_domain=       getOption('smb_domain');

/** "Starting Point" for user id */
$ux_uid=           getOption('ux_uid');

/** "Starting Point" for group id */
$ux_gid=           getOption('ux_gid');

/** The users home directory */
$ux_homedir=       getOption('ux_homedir');

/** 
 * Adds a group to every user and sets its primary group to this group. 
 * Beware that this will break "usrmgr"!
 */
$add_prim_group=   getOption('add_prim_group');

/**
 * Adds Samba Attributes to users primary group.
 * This is only valid with $add_prim_group=true
 */
$add_prim_smb=     getOption('add_prim_smb');

/**
 * Adds new users to group "Domain Users".
 * Only valid if add_prim_group=true
 */
$user_prim_domain= getOption('user_prim_domain');

/** Adds Samba attributes to every new object. This will break "usrmgr" */
$add_samba_attr=   getOption('add_samba_attr');

/** 
 * Verify that RID is not already used. Makes really only sense if 
 * add_samba_attr=true and/or add_prim_smb=true. 
 * You may enable this if you vampired (net rpc vampire) 
 * your domain and have not enough IDs left.
 */
$verify_rid=       getOption('verify_rid');

/** Adds sambaIdMapEntry ObjectClass to samba Accounts. Same prerequisites as $verify_rid */
$add_id_map=       getOption('add_id_map');

/** Hostname of the LDAP-Server */
$ldap_host=        getOption('ldap_host');

/** Base of the LDAP-Tree */
$ldap_base=        getOption('ldap_base');

/** BindDN to use for connection */
$ldap_binddn=      getOption('ldap_binddn');

/** Related password for BindDN */
$ldap_pass=        getOption('ldap_pass');

/** Whether to use TLS to connect to LDAP Server */
$ldap_tls=         getOption('ldap_tls');

/** The port to connect to */
$ldap_port=        getOption('ldap_port');

/** LDAP protocol version to use (Valid values are 1, 2, 3) */
$ldap_prot=        getOption('ldap_prot');

/** Suffix for users. Will be prepended to base, presets to base if not set */
$ldap_suf_user=    getOption('ldap_suf_user');

/** Suffix for groups. Will be prepended to base, presets to base if not set */
$ldap_suf_group=   getOption('ldap_suf_group');

/** Suffix for machines. Will be prepended to base, presets to base if not set */
$ldap_suf_mach=    getOption('ldap_suf_mach');

/** Should a POSIX Account be created */
$create_users=     getOption('create_users');

/** Should the POSIX Account be deleted */
$delete_users=     getOption('delete_users');

/** Should a machine Account be created */
$create_machines=  getOption('create_machines');

/** Should a POSIX Group be created */
$create_groups=    getOption('create_groups');

/** Should the POSIX Group be deleted */
$delete_groups=    getOption('delete_groups');

/** */
$mod_groups=       getOption('mod_groups');

/** Command to be executed after user has been created */
$post_exec_user=   getOption('post_exec_user');

/** Command to be executed after machine account has been created */
$post_exec_mach=   getOption('post_exec_mach');

/** Command to be execeuted after group has been created */
$post_exec_grup=   getOption('post_exec_grup');

/** Program Path to be included */
$prog_path=        getOption('prog_path');

// Read Funtions
require_once("functions.php4");

// Connect to LDAP-Server (this is global)
$ldap_con = ldapConnect();
	
/**
 * Simple function to check if needed OU-base entries are present in LDAP.
 * If not, tries to add these.
 */
function checkOU(){
	global $ldap_base,$ldap_suf_user,$ldap_suf_group,$ldap_suf_mach,$ldap_con;
	
	// Check Variables
	if ($ldap_base == "") {
		deBug("No LDAP-Base was set. Exiting");
		exit(1);
	} else if ( 
		$ldap_suf_user == "" || 
		$ldap_suf_group == "" ||
		$ldap_suf_mach == "") {
			deBug("One of the suffixes is empty. Exiting");
			exit(1);
	}

	// Check for empty LDAP (no base)
	ldapSearch($ldap_con,"(objectClass=*)",getBase());
	if( ldap_error($ldap_con) == "No such object") {
		deBug("Empty Base. Will try to add base entry");
		if(writeBaseEntry()) {
			deBug("Base entry added.");
		} else {
			deBug("Failed to add base entry. Please fix this manually.");
			print("Error was ".ldap_error($ldap_con)."\n");
		}
	}

	// Check for user subtree
	ldapSearch($ldap_con,"(objectClass=*)",getBase($ldap_suf_user));
	if( ldap_error($ldap_con) == "No such object") {
		deBug("No User OU found. Will try to fix this.");
		if(writeOU($ldap_suf_user)) {
			deBug("User OU entry added.");
		} else {
			deBug("Failed to add User OU entry. Please fix this manually.");
			print("Error was ".ldap_error($ldap_con)."\n");
		}
	}

	// Check for group subtree
	ldapSearch($ldap_con,"(objectClass=*)",getBase($ldap_suf_group));
	if( ldap_error($ldap_con) == "No such object") {
		deBug("No Group OU found. Will try to fix this.");
		if(writeOU($ldap_suf_group)) {
			deBug("Group OU entry added.");
		} else {
			deBug("Failed to add Group OU entry. Please fix this manually.");
			print("Error was ".ldap_error($ldap_con)."\n");
		}
	}
	// Check for machine subtree
	ldapSearch($ldap_con,"(objectClass=*)",getBase($ldap_suf_mach));
	if( ldap_error($ldap_con) == "No such object") {
		deBug("No Machine OU found. Will try to fix this.");
		if(writeOU($ldap_suf_mach)) {
			deBug("Machine OU entry added.");
		} else {
			deBug("Failed to add Machine OU entry. Please fix this manually.");
			print("Error was ".ldap_error($ldap_con)."\n");
		}
	}
}

/** Adds Domain SID to LDAP-Tree
 * @param $ldap_con Valid LDAP connection handle
 * @return true if success, false otherwise
 */
function addDomainSid(){
 	global $GDATA,$ldap_base,$smb_domain,$ldap_con;
	// Check for empty LDAP (no base)
	checkOU();

	// First try to get the SID via the 'net' command
	if(isset($smb_domain)) {
		$net_cmd = trim(`which net`);
		if($net_cmd == "") {
			deBug("the 'net' commando was not found in path. please fix");
			exit(1);
		}
		
		$cmd_line = "$net_cmd rpc info -w $smb_domain";
		
		// Convert Multi-Line string to one line
		$string = preg_replace("[\n]", " ", `$cmd_line`);
		
		// TODO: Verify SID-Format
		$regex = "/S\-\d\-\d\-\d{2}\-\d{9}\-\d{10}\-\d{8}/";	
		
		// Save SID in $GDATA["domsid"]
		preg_match($regex, $string, $sid);
		if(isset($sid[0])){
			$GDATA["domsid"] = $sid[0];
			deBug("Will try to add SID: ".$GDATA["domsid"]);
		}

		if(isset($GDATA["domsid"])) {
			$ldapadd["objectclass"][0]="sambaDomain";
			$ldapadd["sambaDomainName"]=$smb_domain;
			$ldapadd["sambaSID"]=$GDATA["domsid"];

			// RidBase is a "May" for sambaDomain
			if(isset($GDATA["ridbase"])) {
				$ldapadd["sambaAlgorithmicRidBase"]=$GDATA["ridbase"];
			}
			return(ldap_add($ldap_con,"sambaDomainName=".$smb_domain.",".$ldap_base,$ldapadd));
		} else {
			deBug("Could not insert base entry");
		}
	}
}

/**
 * Will try to write ou for given suffix
 * @param $organizationalUnit suffix for ou
 * @return true if added, false otherwise
 */
function writeOU($organizationalUnit) {
	global $ldap_base,$ldap_con;
	$local_base = $ldap_base;
	// Check if we need to add multiple entries (as for ou=people,ou=Machines)
	$entries = preg_split("/,/", $organizationalUnit);
	foreach(array_reverse($entries) as $entry) {
		$ou = array_pop(preg_split("/=/", $entry));
		$ldapadd["objectclass"]="organizationalUnit";
		$ldapadd["ou"]=$ou;
		$result = ldap_add($ldap_con,$entry.",".$local_base,$ldapadd);
		if(!$result){
			print("Error was ".ldap_error($ldap_con));
		}
		$local_base = "ou=".$ou.",".$local_base;
	}
	return result;
}

// Writes top Entry for empty LDAP tree
function writeBaseEntry(){
	global $ldap_base,$ldap_con;
	$organization = array_pop(preg_split("/=/", array_shift(preg_split("/,/", $ldap_base))));
	$ldapadd["objectclass"]="organization";
	$ldapadd["o"]=$organization;
	return(ldap_add($ldap_con,$ldap_base,$ldapadd));
}

function checkStandardGroups() {
	global $ldap_base,$ldap_con,$ldap_suf_group,$ldap_suf_mach,$GDATA;
	$group_base = getBase($ldap_suf_group);
	$mach_base = getBase($ldap_suf_mach);

	// First try to find the user entries
	$result = getPrimaryGroup($ldap_con,"users");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-513";
		$ldapadd['cn']="domain users";
		
		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
		ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
	}

	// admins
	$result = getPrimaryGroup($ldap_con,"admins");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-512";
		$ldapadd['cn']="domain admins";

		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
		ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
	}
	 // guests
	$result = getPrimaryGroup($ldap_con,"guests");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-514";
		$ldapadd['cn']="domain guests";
		
		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
		ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
	}

	// machines
	$result = getPrimaryGroup($ldap_con,"machines");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-516";
		$ldapadd['cn']="domain machines";
		
		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
		ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$mach_base,$ldapadd);
	}
}
	
function populate() {
	// First try to write the base entry
	writeBaseEntry();
	
	// Write Domain SID
	getDomainSid();
	
	// Write OUs
	checkOU();
}

// Parse commandline
if ($argc<2) { help(); }
for($i=1;$i<$argc;$i++){
	switch($argv[$i]) {
		/** @cond */
		case "--help":
			help();
			exit;
			break;
			
		case "--populate":
			populate();
			break;
		
		case "--checkou":
			checkOU();
			break;

		case "--checkgroups":
			checkStandardGroups();
			break;
			
		case "--addsid":
			getDomainSid();
			break;

		default:
			help();
			exit(1);
		/** @endcond */
	}
}

function help() {
		$call = "populate.php4";
	
		$help="usage: populate.php4 function ...\n"
		."--help\t\t\tThe text you are reading.\n"
		."--populate\t\tCreates required entries in empty LDAP-Tree.\n"
		."--checkou\t\tChecks for required OU-Entries for saving user,\n\t\t\tgroup and machine accounts and creates them.\n"
		."--checkgroups\t\tChecks for required Windows Group Mappings."
		."--addsid\t\tAdds Domain SID entry to LDAP.\n\t\t\tYou should better use samba directly.\n";
		print $help;
}

// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:
?>
